/* datei.h
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: datei.h,v 1.46 2003/07/09 23:33:20 ralf Exp $ */

#ifndef DATEI_H
#define DATEI_H

#include "wdefines.h"
#include <aguix/util.h>
#include <aguix/fieldlistview.h>
#include <aguix/fieldlistviewdnd.h>
#include "worker_types.h"
#include <vector>

/* vvv Nur eins davon kann gleichzeitig aktiv sein, also durch numerieren */
enum { SORT_NAME = 0,
       SORT_SIZE,
       SORT_ACCTIME,
       SORT_MODTIME,
       SORT_CHGTIME,
       SORT_TYPE,
       SORT_OWNER,
       SORT_INODE,
       SORT_NLINK };

/* for checking valid sortmode */
#define ISVALID_SORTMODE(sortmode) ( ( ( sortmode & 0xff ) >= SORT_NAME ) && ( ( sortmode & 0xff ) <= SORT_NLINK ) )

/* vvv Dient zur Ver-Oderung mit dem Mode */
#define SORT_REVERSE 256
enum {SORT_DIRLAST=512,SORT_DIRMIXED=1024};

// how many characters are used for permission string
// NOT including the trailing null byte
#define WORKER_PERMCHARS 10

class WCFiletype;

class Datei
{
public:
  Datei();
  ~Datei();
  int open( const char *name, const char *mode );
  void close();
  int putUChar(unsigned char value);
  int putUShort(unsigned short int value);
  int putInt(int);
  int putULong(unsigned long value);
  int putString( const char *str1 );
  unsigned char getUChar();
  int getCharAsInt();
  unsigned short int getUShort();
  int getInt();
  unsigned long getULong();
  char *getString(int count);
  int getString(char *,int count);
  void seek(long offset,int mode);
  int overreadChunk();
  long errors();
  int putStringExt(const char *format,const char *str);
  int putLine( const char *str );
  char *getLine();
  bool isEOF();
  
  static int getIntSize();
  static int getUCharSize();
  static int getUShortSize();
  static int getULongSize();
  static bool fileExists(const char*);
  typedef enum {D_FE_NOFILE,D_FE_FILE,D_FE_DIR,D_FE_LINK} d_fe_t;
  static d_fe_t fileExistsExt(const char*);
  static d_fe_t lfileExistsExt(const char*);
  static char *getFilenameFromPath(const char*);
  static char *getNameWithoutExt(const char*);
  static char *createTMPName();
  static char *createTMPName( const char *infix );
  static char *shrinkFilename( const char *str, int maxlength );
  static loff_t fileSize( const char * );
  static char *getRelativePath( const char *source, const char *destdir );
private:
  FILE *fp;
  long error;
};

class FileEntry
{
public:
  FileEntry();
  ~FileEntry();
  bool isDir();
  FileEntry *duplicate();
  int readInfos();
  int readInfos(bool);
  char *getDestination() const;
  int writeDestination2Buf(char*,int,bool);
  const char *getPermissionString();
  bool isExe();
  WCFiletype* checkFiletype( List *ftlist, bool skipcontent );
  bool match(char* pattern);
  bool checkAccess( int m );

  void setLVC( FieldListView *lv, int row, const std::vector<WorkerTypes::listcol_t> *dis );
  void setLVC_DND( FieldListViewDND *lv, int row, const std::vector<WorkerTypes::listcol_t> *dis );
  
  const loff_t size() const;
  const time_t lastaccess() const;
  const time_t lastmod() const;
  const time_t lastchange() const;
  const mode_t mode() const;
  const uid_t userid() const;
  const gid_t groupid() const;
  const ino_t inode() const;
  const nlink_t nlink() const;
  const loff_t blocks() const;
  const loff_t dsize() const;
  const time_t dlastaccess() const;
  const time_t dlastmod() const;
  const time_t dlastchange() const;
  const mode_t dmode() const;
  const uid_t duserid() const;
  const gid_t dgroupid() const;
  const ino_t dinode() const;
  const nlink_t dnlink() const;
  const loff_t dblocks() const;
  const dev_t rdev() const;
  const dev_t drdev() const;

  struct FE_stat {
    loff_t size;
    time_t lastaccess;
    time_t lastmod;
    time_t lastchange;
    mode_t mode;
    uid_t userid;
    gid_t groupid;
    ino_t inode;
    nlink_t nlink;
    loff_t blocks;
    dev_t rdev;
  } statbuf, dstatbuf;

  char *fullname;
  char *name;
  loff_t dirsize;
  bool select;
  bool isLink;
  bool isCorrupt;
  
  long nr;
  bool inFilter;
  WCFiletype *filetype;
  bool isHidden;
  bool use;
  
  bool reclistQueued;  // for filetype recognition in normalmode
};

class Verzeichnis
{
public:
  Verzeichnis();
  ~Verzeichnis();
  ArrayList *getFiles();
  int readDir( const char *, bool dontWarnInvalid = false );
  void closeDir();
  int sort(int);
  int sort(int,bool force);
  int getSize();
  char *getDir();

  int getUseSize();
  static int sortfunction(void*,void*,int);
  ArrayList *getInvalidEntries();
protected:

  ArrayList *files;
  char *actdir;
  int actsortmode;
  
  ArrayList *invalid_entries;
};

char *ParentDir(const char *,char *);
char *ParentDir(const char *,char *,int);
char *HandlePath(const char *);
char *HandlePathExt(const char*);
int getOwnerStringLen(uid_t,gid_t);
char* getOwnerString(uid_t,gid_t);
char *resolveEnv(const char*);

extern int datelen;

int getDatelen();

#endif

